package org.owasp.appsensor;

import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Locale;
import java.util.Set;

import javax.servlet.http.HttpSession;

import org.owasp.esapi.ESAPI;
import org.owasp.esapi.Encoder;
import org.owasp.esapi.errors.AuthenticationException;
import org.owasp.esapi.errors.EncryptionException;

public class AppSensorSystemUser implements ASUser {

	/**
	 * The system user is used to represent an user in the case that an exception 
	 * is thrown that is not in the context of a user, ie. an AppSensorSystemException. 
	 * Since there is always a real user, the system user is better than using null to represent this.
	 */
	private static final long serialVersionUID = 5716176095812125529L;

	private String csrfToken = "";
	private Set<HttpSession> sessions = new HashSet<HttpSession>();
	private Locale locale = null;

	private static final String SYSTEM_USER = "APP_SENSOR_SYSTEM_USER";
	
	/**
	 * {@inheritDoc}
	 */
	public void addRole(String role) throws AuthenticationException {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	public void addRoles(Set<String> newRoles) throws AuthenticationException {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	public void changePassword(String oldPassword, String newPassword1, String newPassword2) throws AuthenticationException, EncryptionException {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	public void disable() {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	public void enable() {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	public long getAccountId() {
		return -100;
	}

	/**
	 * {@inheritDoc}
	 */
	public String getAccountName() {
		return SYSTEM_USER;
	}

	/**
	 * Alias method that is equivalent to getAccountName()
	 * 
	 * @return the name of the current user's account
	 */
	public String getName() {
		return getAccountName();
	}

	/**
	 * {@inheritDoc}
	 */
	public String getCSRFToken() {
		return csrfToken;
	}

	/**
	 * {@inheritDoc}
	 */
	public Date getExpirationTime() {
		return null;
	}

	/**
	 * {@inheritDoc}
	 */
	public int getFailedLoginCount() {
		return 0;
	}

	/**
	 * {@inheritDoc}
	 */
	public Date getLastFailedLoginTime() throws AuthenticationException {
		return null;
	}

	/**
	 * {@inheritDoc}
	 */
	public String getLastHostAddress() {
		return "unknown";
	}

	/**
	 * {@inheritDoc}
	 */
	public Date getLastLoginTime() {
		return null;
	}

	/**
	 * {@inheritDoc}
	 */
	public Date getLastPasswordChangeTime() {
		return null;
	}

	/**
	 * {@inheritDoc}
	 */
	public Set<String> getRoles() {
		return new HashSet<String>();
	}

	/**
	 * {@inheritDoc}
	 */
	public String getScreenName() {
		return SYSTEM_USER;
	}

	/**
	 * {@inheritDoc}
	 */
	public void addSession(HttpSession s) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void removeSession(HttpSession s) {
	}

	/**
	 * {@inheritDoc}
	 */
	public Set<HttpSession> getSessions() {
		return sessions;
	}

	/**
	 * {@inheritDoc}
	 */
	public void incrementFailedLoginCount() {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean isAnonymous() {
		return true;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean isEnabled() {
		return false;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean isExpired() {
		return false;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean isInRole(String role) {
		return false;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean isLocked() {
		return false;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean isLoggedIn() {
		return false;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean isSessionAbsoluteTimeout() {
		return false;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean isSessionTimeout() {
		return false;
	}

	/**
	 * {@inheritDoc}
	 */
	public void lock() {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	public void loginWithPassword(String password) throws AuthenticationException {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	public void logout() {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	public void removeRole(String role) throws AuthenticationException {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	@SuppressWarnings("deprecation")
	public String resetCSRFToken() throws AuthenticationException {
		csrfToken = ESAPI.randomizer().getRandomString(8, Encoder.CHAR_ALPHANUMERICS);
		return csrfToken;
	}

	/**
	 * {@inheritDoc}
	 */
	public void setAccountName(String accountName) {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	public void setExpirationTime(Date expirationTime) {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	public void setRoles(Set<String> roles) throws AuthenticationException {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	public void setScreenName(String screenName) {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	public void unlock() {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean verifyPassword(String password) throws EncryptionException {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	public void setLastFailedLoginTime(Date lastFailedLoginTime) {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	public void setLastLoginTime(Date lastLoginTime) {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	public void setLastHostAddress(String remoteHost) {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	public void setLastPasswordChangeTime(Date lastPasswordChangeTime) {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * {@inheritDoc}
	 */
	@SuppressWarnings("unchecked")
	public HashMap getEventMap() {
		throw new RuntimeException("Invalid operation for the system user");
	}

	/**
	 * @return the locale
	 */
	public Locale getLocale() {
		return locale;
	}

	/**
	 * @param locale
	 *            the locale to set
	 */
	public void setLocale(Locale locale) {
		this.locale = locale;
	}

}
