package org.owasp.appsensor.reference;

import java.util.Hashtable;
import org.owasp.appsensor.AppSensorServiceMonitor;
import org.owasp.appsensor.ServiceStore;

/**
 * NOTE - The locking employed in the class isn't really suitable for a proper multithreaded distributed
 * environment.
 * @author luke b
 *
 */
public class DefaultServiceStore implements ServiceStore {
    
    /** Hashtable of path to AppSensorServiceMonitor object, represented by the request URI in the default implementation */
    private static final Hashtable<String, AppSensorServiceMonitor> serviceStorage = new Hashtable<String, AppSensorServiceMonitor>();
    /** Hashtable of path to AppSensorServiceMonitor object, represented by the request URI / user id in the default implementation */
    private static final Hashtable<String, AppSensorServiceMonitor> perUserServiceStorage = new Hashtable<String, AppSensorServiceMonitor>();
    
    /**
     * Retrieves the AppSensorServiceMonitor object for the service specified, creates an object if doesn't exist
     * @param service name of the service to retrieve 
     * @return AppSensorServiceMonitor object for the service specified
     */
    public AppSensorServiceMonitor getService(final String service) {
        AppSensorServiceMonitor serviceMon = serviceStorage.get(service);
        if (serviceMon == null) {
            serviceMon = new AppSensorServiceMonitor(service, null);
            serviceStorage.put(service, serviceMon);
        }
        return serviceMon;
    }
    
    /**
     * Retrieves the AppSensorServiceMonitor object for the service/userId specified, creates an object if doesn't exist
     * @param service name of the service to retrieve 
     * @param userId userId of the user to retrieve
     * @return AppSensorServiceMonitor object for the service/userId specified
     */
    public AppSensorServiceMonitor getServiceForUser(final String service, final String userId) {
        final String key = getServiceKey(service, userId);
        AppSensorServiceMonitor serviceMon = perUserServiceStorage.get(key);
        if (serviceMon == null) {
            serviceMon = new AppSensorServiceMonitor(service, userId);
            perUserServiceStorage.put(key, serviceMon);
        }
        return serviceMon;
    }
    
    public void save(final AppSensorServiceMonitor assm) {
        // Nothing to do here as this implementation is entirely in process.
        // If this were a distributed system like membase or mysql we'd write to that.
    }
    
    /**
     * Helper method to create concatenated service/user string to represent the 
     * unique key for service/user storage
     * @param service name of the service 
     * @param userId name of the userId
     * @return concatenated service/user string to represent the unique key for service/user storage
     */
    private static String getServiceKey(final String service, final String userId) {
        if (userId != null) {
            return service + "--" + userId;
        }
        return service;
    }
}
